from customtkinter import CTk
from typing import TYPE_CHECKING
from abc import ABC

from .interface import ModelObserver
from .model import ObservableModel

if TYPE_CHECKING:
    from controller import Controller


class View(CTk, ModelObserver):

    '''Base class for views'''

    def __init__(self, controller: 'Controller', model: ObservableModel):

        super().__init__()

        self._controller = controller
        self._model = model

        self._model.add_observer(self)


from abc import ABC
from typing import TYPE_CHECKING

from .model import ObservableModel

if TYPE_CHECKING:
    from view import View


class Controller(ABC):

    '''Base abstract class for controllers'''

    def __init__(self, view_cls: 'type[View]', model: ObservableModel):

        self._view = view_cls(self, model)
        self._model = model

        self._view.mainloop()


from abc import ABC

from .interface import ModelObserver

class ObservableModel(ABC):

    '''Base abstract class for models'''

    def __init__(self):

        self._observers: list[ModelObserver] = []

    def add_observer(self, observer: ModelObserver):
        
        self._observers.append(observer)

    def remove_observer(self, observer: ModelObserver):
        
        self._observers.remove(observer)

    def notify_observers(self):
        
        for observer in self._observers:

            observer.model_is_changed()